@@ -45,7 +45,7 @@ solve(const char *equation)
     MPEquationOptions options;
     MPErrorCode error;
     MPNumber result;
-    char result_str[1024];
+    char *result_str;
 
     memset(&options, 0, sizeof(options));
     options.base = 10;
@@ -62,8 +62,9 @@ solve(const char *equation)
         exit(1);
     }
     else {
-        mp_serializer_to_specific_string(&result, options.base, 9, true, true, result_str, 1024);
+        mp_serializer_to_specific_string(&result, options.base, 9, true, true, &result_str);
         printf("%s\n", result_str);
+        g_free(result_str);
         exit(0);
     }
 }
@@ -213,8 +214,8 @@ main(int argc, char **argv)
     gchar *source_currency, *target_currency;
 
     g_type_init();
-    setlocale(LC_NUMERIC, "");
 
+    setlocale(LC_ALL, "");
     bindtextdomain(GETTEXT_PACKAGE, LOCALE_DIR);
     bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
     textdomain(GETTEXT_PACKAGE);
