@@ -92,10 +92,20 @@ struct MathEquationPrivate
 
     gboolean in_delete;
 
+    gboolean in_solve;
+
     MathVariables *variables;
     MpSerializer *serializer;
+
+    GAsyncQueue *queue;
 };
 
+typedef struct {
+    MPNumber *number_result;
+    gchar *text_result;
+    gchar *error;
+} SolveData;
+
 G_DEFINE_TYPE (MathEquation, math_equation, GTK_TYPE_TEXT_BUFFER);
 
 
@@ -137,13 +147,13 @@ reformat_ans(MathEquation *equation)
         return;
 
     gchar *orig_ans_text;
-    gchar ans_text[MAX_DIGITS];
+    gchar *ans_text;
     GtkTextIter ans_start, ans_end;
 
     gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(equation), &ans_start, equation->priv->ans_start);
     gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(equation), &ans_end, equation->priv->ans_end);
     orig_ans_text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(equation), &ans_start, &ans_end, FALSE);
-    mp_serializer_to_standard_string(equation->priv->serializer, &equation->priv->state.ans, ans_text, MAX_DIGITS);
+    mp_serializer_to_standard_string(equation->priv->serializer, &equation->priv->state.ans, &ans_text);
     if (strcmp(orig_ans_text, ans_text) != 0) {
         gint start;
 
@@ -165,168 +175,16 @@ reformat_ans(MathEquation *equation)
     gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(equation), &ans_start, equation->priv->ans_start);
     gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(equation), &ans_end, equation->priv->ans_end);
     g_free(orig_ans_text);
+    g_free(ans_text);
 }
 
-#if 0
-/* NOTE: Not efficent but easy to write */
-// FIXME: This is just a lexer - use the same lexer as the solver
-static void
-reformat_base(MathEquation *equation, gint old_base)
-{
-    gunichar sub_zero, sub_nine;
-    gchar *text, *read_iter;
-    gboolean in_number = FALSE, have_radix = FALSE;
-    gint offset = 0, offset_step = 0, max_digit = 0, base = -1, base_offset = 0;
-    gint ans_start, ans_end;
-
-    if (equation->priv->base == old_base)
-        return;
-
-    sub_zero = g_utf8_get_char("₀");
-    sub_nine = g_utf8_get_char("₉");
-
-    read_iter = text = math_equation_get_display(equation);
-    get_ans_offsets(equation, &ans_start, &ans_end);
-    while (TRUE) {
-        gunichar c;
-        gint digit = -1, sub_digit = -1;
-
-        /* See what digit this character is */
-        c = g_utf8_get_char(read_iter);
-        if (c >= sub_zero && c <= sub_nine)
-            sub_digit = c - sub_zero;
-        else if (c >= 'a' && c <= 'z')
-            digit = c - 'a';
-        else if (c >= 'A' && c <= 'Z')
-            digit = c - 'A';
-        else
-            digit = g_unichar_digit_value(c);
-
-        /* Don't mess with ans */
-        if (offset >= ans_start && offset <= ans_end) {
-            digit = -1;
-            sub_digit = -1;
-        }
-
-        if (in_number && digit >= 0) {
-            if (digit > max_digit)
-                max_digit = digit;
-        }
-        else if (in_number && sub_digit >= 0) {
-            if (base < 0) {
-                base_offset = offset;
-                base = 0;
-            }
-
-            base = base * 10 + sub_digit;
-        }
-        else if (in_number) {
-            /* Allow one radix inside a number */
-            if (!have_radix && base < 0 && strncmp(read_iter, equation->priv->radix, strlen(equation->priv->radix)) == 0) {
-                have_radix = TRUE;
-                read_iter += strlen(equation->priv->radix);
-                offset += g_utf8_strlen(equation->priv->radix, -1);
-                continue;
-            }
-
-            /* If had no base then insert it */
-            if (base < 0) {
-                GtkTextIter iter;
-                gint multiplier = 1;
-                gint b = old_base;
-                const char *digits[] = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉"};
-
-                equation->priv->in_undo_operation = TRUE;
-                equation->priv->in_reformat = TRUE;
-
-                while (b / multiplier != 0)
-                    multiplier *= 10;
-                while (multiplier != 1) {
-                    int d;
-                    multiplier /= 10;
-                    d = b / multiplier;
-                    gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(equation), &iter, offset + offset_step);
-                    gtk_text_buffer_insert(GTK_TEXT_BUFFER(equation), &iter, digits[d], -1);
-                    offset_step++;
-                    b -= d * multiplier;
-                }
-
-                equation->priv->in_reformat = FALSE;
-                equation->priv->in_undo_operation = FALSE;
-            }
-            /* Remove the base if the current value */
-            else if (max_digit < base && base == equation->priv->base) {
-                GtkTextIter start, end;
-
-                equation->priv->in_undo_operation = TRUE;
-                equation->priv->in_reformat = TRUE;
-
-                gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(equation), &start, base_offset + offset_step);
-                gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(equation), &end, offset + offset_step);
-                gtk_text_buffer_delete(GTK_TEXT_BUFFER(equation), &start, &end);
-                offset_step -= offset - base_offset;
-
-                equation->priv->in_reformat = FALSE;
-                equation->priv->in_undo_operation = FALSE;
-            }
-
-            in_number = FALSE;
-        }
-        else if (digit >= 0) {
-            in_number = TRUE;
-            have_radix = FALSE;
-            base = -1;
-            max_digit = digit;
-        }
-
-        if (c == '\0')
-            break;
-               
-        read_iter = g_utf8_next_char(read_iter);
-        offset++;
-    }
-
-    g_free(text);
-}
-
-
-static void
-reformat_separators(MathEquation *equation)
-{
-#if 0
-    gchar *text, *read_iter;
-    gboolean in_number = FALSE, in_fraction = FALSE;
-
-    text = math_equation_get_display(equation);
-
-    /* Find numbers in display, and modify if necessary */
-    read_iter = text;
-    while(*read_iter != '\0') {
-        gunichar c;
-      
-        c = g_utf8_get_char(read_iter);
-
-        if (strncmp(read_iter, equation->priv->tsep, strlen(equation->priv->tsep)) == 0)
-            ;
-        read_iter = g_utf8_next_char(read_iter);
-    }
-
-    g_free(text);
-#endif
-}
-#endif
-
 static void
 reformat_display(MathEquation *equation, gint old_base)
 {
     /* Change ans */
     reformat_ans(equation);
 
-    /* Add/remove base suffixes if have changed base */
-    //reformat_base(equation, old_base);
-
-    /* Add/remove thousands separators */
-    //reformat_separators(equation);
+    g_signal_emit_by_name(equation, "answer-changed");
 }
 
 
@@ -819,6 +677,13 @@ math_equation_get_number_mode(MathEquation *equation)
 }
 
 
+gboolean
+math_equation_in_solve(MathEquation *equation)
+{
+    return equation->priv->in_solve;
+}
+
+
 const MPNumber *
 math_equation_get_answer(MathEquation *equation)
 {
@@ -857,11 +722,11 @@ math_equation_set(MathEquation *equation, const gchar *text)
 void
 math_equation_set_number(MathEquation *equation, const MPNumber *x)
 {
-    char text[MAX_DIGITS];
+    char *text;
     GtkTextIter start, end;
 
     /* Show the number in the user chosen format */
-    mp_serializer_to_standard_string(equation->priv->serializer, x, text, MAX_DIGITS);
+    mp_serializer_to_standard_string(equation->priv->serializer, x, &text);
     gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), text, -1);
     mp_set_from_mp(x, &equation->priv->state.ans);
 
@@ -871,6 +736,7 @@ math_equation_set_number(MathEquation *equation, const MPNumber *x)
     equation->priv->ans_start = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(equation), NULL, &start, FALSE);
     equation->priv->ans_end = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(equation), NULL, &end, TRUE);
     gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(equation), equation->priv->ans_tag, &start, &end);
+    g_free(text);
 }
 
 
@@ -900,8 +766,6 @@ math_equation_insert(MathEquation *equation, const gchar *text)
     if (strstr("⁻⁰¹²³⁴⁵⁶⁷⁸⁹₀₁₂₃₄₅₆₇₈₉", text) == NULL)
         math_equation_set_number_mode(equation, NORMAL);
 
-    // FIXME: Add thousands separators
-
     gtk_text_buffer_delete_selection(GTK_TEXT_BUFFER(equation), FALSE, FALSE);
     gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(equation), text, -1);
 }
@@ -932,9 +796,10 @@ math_equation_insert_numeric_point(MathEquation *equation)
 void
 math_equation_insert_number(MathEquation *equation, const MPNumber *x)
 {
-    char text[MAX_DIGITS];
-    mp_serializer_to_standard_string(equation->priv->serializer, x, text, MAX_DIGITS);
+    char *text;
+    mp_serializer_to_standard_string(equation->priv->serializer, x, &text);
     math_equation_insert(equation, text);
+    g_free(text);
 }
 
 
@@ -1045,26 +910,20 @@ parse(MathEquation *equation, const char *text, MPNumber *z, char **error_token)
     return mp_equation_parse(text, &options, z, error_token);
 }
 
-
-void
-math_equation_solve(MathEquation *equation)
+/*
+ * Executed in separate thread. It is thus not a good idea to write to anything
+ * in MathEquation but the async queue from here.
+ */
+static gpointer
+math_equation_solve_real(gpointer data)
 {
-    MPNumber z;
-    gint result, n_brackets = 0;
-    gchar *c, *text, *error_token = NULL, *message = NULL;
+    MathEquation *equation = MATH_EQUATION(data);
+    SolveData *solvedata = g_slice_new0(SolveData);
+
+    gint n_brackets = 0, result;
+    gchar *c, *text, *error_token;
     GString *equation_text;
-
-    if (math_equation_is_empty(equation))
-        return;
-
-    /* If showing a result return to the equation that caused it */
-    // FIXME: Result may not be here due to solve (i.e. the user may have entered "ans")
-    if (math_equation_is_result(equation)) {
-        math_equation_undo(equation);
-        return;
-    }
-
-    math_equation_set_number_mode(equation, NORMAL);
+    MPNumber z;
 
     text = math_equation_get_equation(equation);
     equation_text = g_string_new(text);
@@ -1080,88 +939,185 @@ math_equation_solve(MathEquation *equation)
         g_string_append_c(equation_text, ')');
         n_brackets--;
     }
-  
+
 
     result = parse(equation, equation_text->str, &z, &error_token);
     g_string_free(equation_text, TRUE);
 
     switch (result) {
         case PARSER_ERR_NONE:
-            math_equation_set_number(equation, &z);
+            solvedata->number_result = g_slice_new(MPNumber);
+            mp_set_from_mp(&z, solvedata->number_result);
             break;
 
         case PARSER_ERR_OVERFLOW:
-            message = g_strdup(/* Error displayed to user when they perform a bitwise operation on numbers greater than the current word */
+            solvedata->error = g_strdup(/* Error displayed to user when they perform a bitwise operation on numbers greater than the current word */
                                _("Overflow. Try a bigger word size"));
             break;
 
         case PARSER_ERR_UNKNOWN_VARIABLE:
-            message = g_strdup_printf(/* Error displayed to user when they an unknown variable is entered */
+            solvedata->error = g_strdup_printf(/* Error displayed to user when they an unknown variable is entered */
                                       _("Unknown variable '%s'"), error_token);
             break;
 
         case PARSER_ERR_UNKNOWN_FUNCTION:
-            message = g_strdup_printf(/* Error displayed to user when an unknown function is entered */
+            solvedata->error = g_strdup_printf(/* Error displayed to user when an unknown function is entered */
                                       _("Function '%s' is not defined"), error_token);
             break;
 
         case PARSER_ERR_UNKNOWN_CONVERSION:
-            message = g_strdup(/* Error displayed to user when an conversion with unknown units is attempted */
+            solvedata->error = g_strdup(/* Error displayed to user when an conversion with unknown units is attempted */
                                _("Unknown conversion"));
             break;
 
         case PARSER_ERR_MP:
-            message = g_strdup(mp_get_error());
+            solvedata->error = g_strdup(mp_get_error());
             break;
 
         default:
-            message = g_strdup(/* Error displayed to user when they enter an invalid calculation */
+            solvedata->error = g_strdup(/* Error displayed to user when they enter an invalid calculation */
                                _("Malformed expression"));
             break;
     }
+    g_async_queue_push(equation->priv->queue, solvedata);
 
-    if (error_token)
-        free(error_token);
-  
-    if (message) {
-        math_equation_set_status(equation, message);
-        g_free(message);
-    }
+    return NULL;
 }
 
+static gboolean
+math_equation_show_in_progress(gpointer data)
+{
+    MathEquation *equation = MATH_EQUATION(data);
+    if (equation->priv->in_solve)
+        math_equation_set_status(equation, "Calculating");
+    return false;
+}
+
+static gboolean
+math_equation_look_for_answer(gpointer data)
+{
+    MathEquation *equation = MATH_EQUATION(data);
+    SolveData *result = g_async_queue_try_pop(equation->priv->queue);
+
+    if (result == NULL)
+        return true;
+
+    equation->priv->in_solve = false;
+
+    if (!result->error)
+        math_equation_set_status(equation, "");
+
+    if (result->error != NULL) {
+        math_equation_set_status(equation, result->error);
+        g_free(result->error);
+    }
+    else if (result->number_result != NULL) {
+        math_equation_set_number(equation, result->number_result);
+        g_slice_free(MPNumber, result->number_result);
+    }
+    else if (result->text_result != NULL) {
+        math_equation_set(equation, result->text_result);
+        g_free(result->text_result);
+    }
+    g_slice_free(SolveData, result);
+
+    g_signal_emit_by_name(equation, "answer-changed");
+    return false;
+}
+
+void
+math_equation_solve(MathEquation *equation)
+{
+    GError *error = NULL;
+
+    // FIXME: should replace calculation or give error message
+    if (equation->priv->in_solve)
+        return;
+
+    if (math_equation_is_empty(equation))
+        return;
+
+    /* If showing a result return to the equation that caused it */
+    // FIXME: Result may not be here due to solve (i.e. the user may have entered "ans")
+    if (math_equation_is_result(equation)) {
+        math_equation_undo(equation);
+        return;
+    }
+
+    equation->priv->in_solve = true;
+
+    math_equation_set_number_mode(equation, NORMAL);
+
+    g_thread_create(math_equation_solve_real, equation, false, &error);
+
+    if (error)
+        g_warning("Error spawning thread for calculations: %s\n", error->message);
+
+    g_timeout_add(50, math_equation_look_for_answer, equation);
+    g_timeout_add(100, math_equation_show_in_progress, equation);
+}
+
+
+static gpointer
+math_equation_factorize_real(gpointer data)
+{
+    GString *text;
+    GList *factors, *factor;
+    MPNumber x;
+    MathEquation *equation = MATH_EQUATION(data);
+    SolveData *result = g_slice_new0(SolveData);
+
+    math_equation_get_number(equation, &x);
+    factors = mp_factorize(&x);
+
+    text = g_string_new("");
+
+    for (factor = factors; factor; factor = factor->next) {
+        gchar *temp;
+        MPNumber *n;
+
+        n = factor->data;
+        mp_serializer_to_standard_string(equation->priv->serializer, n, &temp);
+        g_string_append(text, temp);
+        if (factor->next)
+            g_string_append(text, "×");
+        g_slice_free(MPNumber, n);
+        g_free(temp);
+    }
+    g_list_free(factors);
+
+    result->text_result = g_strndup(text->str, text->len);
+    g_async_queue_push(equation->priv->queue, result);
+    g_string_free(text, TRUE);
+
+    return NULL;
+}
 
 void
 math_equation_factorize(MathEquation *equation)
 {
     MPNumber x;
-    GList *factors, *factor;
-    GString *text;
-  
+    GError *error = NULL;
+
+    // FIXME: should replace calculation or give error message
+    if (equation->priv->in_solve)
+        return;
+
     if (!math_equation_get_number(equation, &x) || !mp_is_integer(&x)) {
         /* Error displayed when trying to factorize a non-integer value */
         math_equation_set_status(equation, _("Need an integer to factorize"));
         return;
     }
 
-    factors = mp_factorize(&x);
+    equation->priv->in_solve = true;
 
-    text = g_string_new("");
+    g_thread_create(math_equation_factorize_real, equation, false, &error);
 
-    for (factor = factors; factor; factor = factor->next) {
-        gchar temp[MAX_DIGITS];
-        MPNumber *n;
+    if (error)
+        g_warning("Error spawning thread for calculations: %s\n", error->message);
 
-        n = factor->data;
-        mp_serializer_to_standard_string(equation->priv->serializer, n, temp, MAX_DIGITS);
-        g_string_append(text, temp);
-        if (factor->next)
-            g_string_append(text, "×");
-        g_slice_free(MPNumber, n);
-    }
-    g_list_free(factors);
-
-    math_equation_set(equation, text->str);
-    g_string_free(text, TRUE);
+    g_timeout_add(50, math_equation_look_for_answer, equation);
+    g_timeout_add(100, math_equation_show_in_progress, equation);
 }
 
 
@@ -1376,6 +1332,18 @@ math_equation_get_property(GObject    *object,
     }
 }
 
+static void
+math_equation_constructed (GObject *object)
+{
+    GtkTextBuffer *parent_class;
+    parent_class = g_type_class_peek_parent(MATH_EQUATION_GET_CLASS(object));
+    if (G_OBJECT_CLASS(parent_class)->constructed)
+        G_OBJECT_CLASS(parent_class)->constructed(object);
+
+    MATH_EQUATION(object)->priv->ans_tag = gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(object), NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
+}
+
+
 
 static void
 math_equation_class_init (MathEquationClass *klass)
@@ -1398,6 +1366,7 @@ math_equation_class_init (MathEquationClass *klass)
 
     object_class->get_property = math_equation_get_property;
     object_class->set_property = math_equation_set_property;
+    object_class->constructed = math_equation_constructed;
 
     g_type_class_add_private (klass, sizeof (MathEquationPrivate));
   
@@ -1506,6 +1475,16 @@ math_equation_class_init (MathEquationClass *klass)
                                                         "Serializer",
                                                         MP_TYPE_SERIALIZER,
                                                         G_PARAM_READABLE));
+
+    g_signal_new ("answer-changed",
+                  G_TYPE_FROM_CLASS(object_class),
+                  G_SIGNAL_RUN_FIRST,
+                  0,
+                  NULL,
+                  NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE,
+                  0);
 }
 
 
@@ -1529,7 +1508,8 @@ pre_insert_text_cb (MathEquation  *equation,
     /* Clear result on next digit entered if cursor at end of line */
     // FIXME Cursor
     c = g_utf8_get_char(text);
-    if (g_unichar_isdigit(c) && math_equation_is_result(equation)) {
+    if ((g_unichar_isdigit(c) || mp_serializer_is_numeric_point(equation->priv->serializer, text)) &&
+            math_equation_is_result(equation)) {
         gtk_text_buffer_set_text(GTK_TEXT_BUFFER(equation), "", -1);
         clear_ans(equation, FALSE);
         gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(equation), location);
@@ -1610,6 +1590,7 @@ delete_range_cb (MathEquation  *equation,
     if (equation->priv->in_reformat)
         return;
 
+    equation->priv->state.entered_multiply = FALSE;
     // FIXME: A replace will emit this both for delete-range and insert-text, can it be avoided?
     g_object_notify(G_OBJECT(equation), "display");
 }
@@ -1627,10 +1608,6 @@ math_equation_init(MathEquation *equation)
 
     equation->priv = G_TYPE_INSTANCE_GET_PRIVATE (equation, math_equation_get_type(), MathEquationPrivate);
 
-    // FIXME: Causes error
-    // (process:18573): Gtk-CRITICAL **: set_table: assertion buffer->tag_table == NULL' failed
-    equation->priv->ans_tag = gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(equation), NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
-
     g_signal_connect(equation, "insert-text", G_CALLBACK(pre_insert_text_cb), equation);
     g_signal_connect(equation, "delete-range", G_CALLBACK(pre_delete_range_cb), equation);  
     g_signal_connect_after(equation, "insert-text", G_CALLBACK(insert_text_cb), equation);
@@ -1656,6 +1633,7 @@ math_equation_init(MathEquation *equation)
     equation->priv->source_currency = g_strdup(currency_names[0].short_name);
     equation->priv->target_currency = g_strdup(currency_names[0].short_name);
     equation->priv->serializer = mp_serializer_new();
+    equation->priv->queue = g_async_queue_new();
 
     mp_set_from_integer(0, &equation->priv->state.ans);
 }
