@@ -187,9 +187,10 @@ static void
 about_cb(GtkWidget *widget, MathWindow *window)
 {
     const gchar *authors[] = {
-        "Rich Burridge <rich.burridge@sun.com>",
+        "Rich Burridge <rich.burridge@gmail.com>",
         "Robert Ancell <robert.ancell@gmail.com>",
         "Klaus Niederkrüger <kniederk@umpa.ens-lyon.fr>",
+        "Robin Sonefors <ozamosi@flukkost.nu>",
         NULL
     };
     const gchar *documenters[] = {
@@ -429,7 +430,7 @@ create_gui(MathWindow *window)
     gtk_widget_show(scrolled_window);
 
     window->priv->display = math_display_new_with_equation(window->priv->equation);
-    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), GTK_WIDGET(window->priv->display));
+    gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(window->priv->display));
     gtk_widget_show(GTK_WIDGET(window->priv->display));
 
     window->priv->buttons = math_buttons_new(window->priv->equation);
